@@ -18,7 +18,6 @@
 
 #include "GxRackTuner.h"
 #include <gtk/gtkmarshal.h>
-#include <gtk/gtkprivate.h>
 #include <math.h>
 #include <string.h>
 
@@ -65,7 +64,6 @@ G_DEFINE_TYPE(GxRackTuner, gx_rack_tuner, GX_TYPE_TUNER);
 
 static void gx_rack_tuner_map(GtkWidget*);
 static void gx_rack_tuner_unmap(GtkWidget*);
-static gboolean gx_rack_tuner_visibility_notify_event(GtkWidget*, GdkEventVisibility*);
 static void gx_rack_tuner_state_changed(GtkWidget*, GtkStateType);
 static gboolean gx_rack_tuner_freq_poll_handler(gpointer);
 static gboolean gx_rack_tuner_configure_event(GtkWidget*, GdkEventConfigure*);
@@ -83,7 +81,6 @@ static void gx_rack_tuner_class_init(GxRackTunerClass *klass)
 	widget_class->size_request = gx_rack_tuner_size_request;
 	widget_class->map = gx_rack_tuner_map;
 	widget_class->unmap = gx_rack_tuner_unmap;
-	widget_class->visibility_notify_event = gx_rack_tuner_visibility_notify_event;
 	widget_class->state_changed = gx_rack_tuner_state_changed;
 	widget_class->configure_event = gx_rack_tuner_configure_event;
 	klass->frequency_poll = 0;
@@ -94,37 +91,37 @@ static void gx_rack_tuner_class_init(GxRackTunerClass *klass)
 		gobject_class, PROP_FREQ, g_param_spec_double (
 			"freq", P_("Frequency"),
 			P_("The frequency for which tuning is displayed"),
-			0.0, 1000.0, 0.0, GParamFlags(GTK_PARAM_READWRITE)));
+			0.0, 1000.0, 0.0, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_SCALE_LIM, g_param_spec_double (
 			"scale-lim", P_("Scale Limit"),
 			P_("The threshold for display of tune match (0.01 = 1 cent)"),
-			0.005, 0.3, 0.03, GParamFlags(GTK_PARAM_READWRITE)));
+			0.005, 0.3, 0.03, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_SPEED, g_param_spec_double (
 			"speed", P_("Streaming Speed"),
 			P_("Speed of streaming mode display (led positions per msec)"),
-			0.0001, 1.0, 0.15, GParamFlags(GTK_PARAM_READWRITE)));
+			0.0001, 1.0, 0.15, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_DISPLAY_FLAT, g_param_spec_boolean (
 			"display-flat", P_("Display Flat Notes"),
 			P_("select flat enharmonic equivalents instead of sharp"),
-			FALSE, GParamFlags(GTK_PARAM_READWRITE)));
+			FALSE, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_STREAMING, g_param_spec_boolean (
 			"streaming", P_("Streaming"),
 			P_("use streaming display"),
-			FALSE, GParamFlags(GTK_PARAM_READWRITE)));
+			FALSE, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_TIMESTEP, g_param_spec_int (
 			"timestep", P_("Time Step"),
 			P_("time interval in msec for refreshing the streaming display"),
-			1, 1000, 50, GParamFlags(GTK_PARAM_READWRITE)));
+			1, 1000, 50, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_LIMIT_TIMESTEP, g_param_spec_int (
 			"limit-timestep", P_("In-Limit Time Step"),
 			P_("time interval in msec for refreshing the streaming match display"),
-			1, 100, 8, GParamFlags(GTK_PARAM_READWRITE)));
+			1, 100, 8, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	/* new signals */
 	signals[FREQUENCY_POLL] =
 		g_signal_new(I_("frequency-poll"),
@@ -173,11 +170,8 @@ static void gx_rack_tuner_init (GxRackTuner *tuner)
 	tuner->strng = 0;
 	// caculated layout
 	tuner->led_count = 0;
-    tuner->width = 0;
-    tuner->padding = 0;
-
-	// visibility-notify-event event needs BUTTON_MOTION instead of VISIBILITY_NOTIFY??
-	gtk_widget_add_events(GTK_WIDGET(tuner), GDK_BUTTON_MOTION_MASK|GDK_VISIBILITY_NOTIFY_MASK);
+	tuner->width = 0;
+	tuner->padding = 0;
 }
 
 static void gx_rack_tuner_destroy(GtkObject *object)
@@ -449,8 +443,8 @@ static void gx_rack_tuner_check_poll(GxRackTuner *tuner)
 
 static void gx_rack_tuner_map(GtkWidget *widget)
 {
-	gx_rack_tuner_check_poll(GX_RACK_TUNER(widget));
 	GTK_WIDGET_CLASS(gx_rack_tuner_parent_class)->map(widget);
+	gx_rack_tuner_check_poll(GX_RACK_TUNER(widget));
 }
 
 static void gx_rack_tuner_unmap(GtkWidget *widget)
@@ -459,16 +453,6 @@ static void gx_rack_tuner_unmap(GtkWidget *widget)
 	gx_rack_tuner_remove_handler(GX_RACK_TUNER(widget));
 }
 
-static gboolean gx_rack_tuner_visibility_notify_event(GtkWidget *widget, GdkEventVisibility *ev)
-{
-	if (ev->state == GDK_VISIBILITY_FULLY_OBSCURED) {
-		gx_rack_tuner_remove_handler(GX_RACK_TUNER(widget));
-	} else {
-		gx_rack_tuner_check_poll(GX_RACK_TUNER(widget));
-	}
-	return FALSE;
-}
-
 static void gx_rack_tuner_state_changed(GtkWidget *widget, GtkStateType oldstate)
 {
 	gx_rack_tuner_check_poll(GX_RACK_TUNER(widget));
