@@ -199,7 +199,8 @@ GxEngine::GxEngine(const string& plugin_dir, ParameterGroups& groups, const gx_s
       preamp(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       loop(get_param(), sigc::mem_fun(mono_chain,&MonoModuleChain::sync),options.get_loop_dir()),
-      record(*this, 1), record_st(*this, 2) {
+      record(*this, 1), record_st(*this, 2),
+      detune(get_param(), *this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync)) {
     set_overload_interval(options.get_sporadic_overload());
     if (!options.get_convolver_watchdog()) {
 	ov_disabled |= ov_Convolver;
@@ -238,6 +239,7 @@ GxEngine::GxEngine(const string& plugin_dir, ParameterGroups& groups, const gx_s
 }
 
 GxEngine::~GxEngine() {
+    pluginlist.cleanup();
 }
 
 void GxEngine::load_static_plugins() {
@@ -290,6 +292,7 @@ void GxEngine::load_static_plugins() {
     pl.add(&crybaby.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&loop.plugin,                          PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&record.plugin,                        PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&detune.plugin,                        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::gx_distortion::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::ts9sim::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::impulseresponse::plugin(), PLUGIN_POS_RACK, PGN_GUI);
@@ -326,6 +329,7 @@ void GxEngine::load_static_plugins() {
     pl.add(pluginlib::gcb_95::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
 	pl.add(gx_effects::duck_delay::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
 	pl.add(pluginlib::reversedelay::plugin(),     PLUGIN_POS_RACK);
+	pl.add(gx_effects::baxandall::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     // stereo
     pl.add(gx_effects::chorus::plugin(),          PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::flanger::plugin(),         PLUGIN_POS_RACK, PGN_GUI);
@@ -351,8 +355,14 @@ void GxEngine::load_static_plugins() {
 
 static LadspaLoader::pluginarray::iterator find_plugin(LadspaLoader::pluginarray& ml, plugdesc *pl) {
     for (LadspaLoader::pluginarray::iterator i = ml.begin(); i != ml.end(); ++i) {
-	if ((*i)->UniqueID == pl->UniqueID) {
-	    return i;
+	if (pl->quirks & is_lv2) {
+	    if ((*i)->path == pl->path) {
+		return i;
+	    }
+	} else {
+	    if ((*i)->UniqueID == pl->UniqueID) {
+		return i;
+	    }
 	}
     }
     return ml.end();
@@ -402,7 +412,7 @@ void GxEngine::ladspaloader_update_plugins() {
     }
     // add new plugins (engine)
     for (LadspaLoader::pluginarray::const_iterator i = ml.begin(); i != ml.end(); ++i) {
-	if (ladspaloader.find((*i)->UniqueID) == ladspaloader.end()) {
+	if (ladspaloader.find(*i) == ladspaloader.end()) {
 	    PluginDef *plugin = ladspaloader.create(*i);
 	    if (plugin) {
 		Plugin *p = pluginlist.add(plugin);
