@@ -1102,7 +1102,7 @@ void ContrastConvolver::run_contrast(int count, float *input0, float *output0, P
 }
 
 /****************************************************************
- ** class LiveLooper
+ ** class smbPitchShift
  */
 
 #include "gx_livelooper.cc"
@@ -1113,4 +1113,518 @@ void ContrastConvolver::run_contrast(int count, float *input0, float *output0, P
 
 #include "gx_record.cc"
 
+
+/****************************************************************************
+*
+* NAME: smbPitchShift.cpp
+* VERSION: 1.2
+* HOME URL: http://www.dspdimension.com
+* KNOWN BUGS: none
+* 
+*
+* COPYRIGHT 1999-2009 Stephan M. Bernsee <smb [AT] dspdimension [DOT] com>
+* 
+* Modified for guitarix by Hermann Meyer 2014
+*
+* 						The Wide Open License (WOL)
+*
+* Permission to use, copy, modify, distribute and sell this software and its
+* documentation for any purpose is hereby granted without fee, provided that
+* the above copyright notice and this license appear in all source copies. 
+* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF
+* ANY KIND. See http://www.dspguru.com/wol.htm for more information.
+*
+*****************************************************************************/ 
+
+bool smbPitchShift::setParameters(int sampleRate_)
+{
+    numSampsToProcess = int(engine.get_buffersize());
+    fftFrameSize = numSampsToProcess/4;
+    sampleRate = int(sampleRate_);
+    assert(sampleRate>0);
+    osamp = 8;
+    osamp1 = 1./osamp;
+    osamp2 = 2.*M_PI*osamp1;
+    mpi = (1./(2.*M_PI)) * osamp;
+    mpi1 = 1./M_PI;
+    fpb = 0; 
+    expect = 0; 
+    hanning = 0; 
+    hanningd = 0;
+    resampin = 0;
+    resampout = 0;
+    indata2 = 0;
+    ftPlanForward = 0;
+    ftPlanInverse = 0;
+    resamp.setup(sampleRate,4);
+    mem_allocated = false;
+    gRover = inFifoLatency;
+    return true;
+}
+
+smbPitchShift::smbPitchShift(ParamMap& param_, EngineControl& engine_, sigc::slot<void> sync_):
+  PluginDef(),
+  engine(engine_),
+  sync(sync_),
+  ready(false),
+  param(param_),
+  plugin() {
+    memset(gInFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gOutFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gLastPhase, 0, (MAX_FRAME_LENGTH/2+1)*sizeof(float));
+    memset(gSumPhase, 0, (MAX_FRAME_LENGTH/2+1)*sizeof(float));
+    memset(gOutputAccum, 0, 2*MAX_FRAME_LENGTH*sizeof(float));
+    memset(gAnaFreq, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gAnaMagn, 0, MAX_FRAME_LENGTH*sizeof(float));
+    version = PLUGINDEF_VERSION;
+    id = "smbPitchShift";
+    name = N_("Detune");
+    groups = 0;
+    description = N_("detune and pitch shift up"); // description (tooltip)
+    category = N_("Misc");       // category
+    mono_audio = compute_static;
+    stereo_audio = 0;
+    set_samplerate = init;
+    activate_plugin = activate_static;
+    register_params = registerparam;
+    delete_instance = del_instance;
+    load_ui = load_ui_f_static;
+    plugin = this;
+    engine.signal_buffersize_change().connect(
+    sigc::mem_fun(*this, &smbPitchShift::change_buffersize));
+}
+
+void smbPitchShift::init(unsigned int samplingFreq, PluginDef *plugin) {
+    static_cast<smbPitchShift*>(plugin)->setParameters(samplingFreq);
+    
+}
+
+void smbPitchShift::clear_state()
+{
+    stepSize = fftFrameSize/osamp;
+    freqPerBin = (double)(sampleRate/4)/(double)fftFrameSize;
+    freqPerBin1 = (1/freqPerBin)*osamp2;
+    freqPerBin2 = freqPerBin*mpi;
+    expct = 2.*M_PI*(double)stepSize/(double)fftFrameSize;
+    inFifoLatency = fftFrameSize-stepSize;
+    fftFrameSize3 = 2. * (1./ ((double)(fftFrameSize2)*osamp));
+    fftFrameSize4 = 1./(double)fftFrameSize;
+    ai = 0;
+    aio = 0;
+    ii = 0;
+    tone =0;
+    memset(gInFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gOutFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gLastPhase, 0, (MAX_FRAME_LENGTH/2+1)*sizeof(float));
+    memset(gSumPhase, 0, (MAX_FRAME_LENGTH/2+1)*sizeof(float));
+    memset(gOutputAccum, 0, 2*MAX_FRAME_LENGTH*sizeof(float));
+    memset(gAnaFreq, 0, MAX_FRAME_LENGTH*sizeof(float));
+    memset(gAnaMagn, 0, MAX_FRAME_LENGTH*sizeof(float));
+    for (k = 0; k < fftFrameSize2;k++) {
+        fpb[k] = (double)k*freqPerBin;
+    }
+    for (k = 0; k < fftFrameSize2;k++) {
+        expect[k] = (double)k*expct;
+    }
+    for (k = 0; k < fftFrameSize;k++) {
+        hanning[k] = 0.5*(1-cos(2.*M_PI*(double)k/((double)fftFrameSize)));
+    }
+    for (k = 0; k < fftFrameSize;k++) {
+        hanningd[k] = 0.5*(1-cos(2.*M_PI*(double)k * fftFrameSize4)) * fftFrameSize3; 
+    }
+    for (k = 0; k < fftFrameSize;k++) {
+        resampin[k] = 0.0; 
+    }
+    for (k = 0; k < fftFrameSize;k++) {
+        resampin2[k] = 0.0; 
+    }
+    for (k = 0; k < fftFrameSize*4;k++) {
+        resampout[k] = 0.0; 
+    }
+    for (k = 0; k < fftFrameSize*4;k++) {
+        indata2[k] = 0.0; 
+    }
+    gRover = inFifoLatency;
+    mem_allocated = true;
+    ready = true;
+}
+
+void smbPitchShift::mem_alloc()
+{
+    numSampsToProcess = int(engine.get_buffersize());
+    assert(numSampsToProcess>0);
+    numSampsToResamp = numSampsToProcess/4;
+    sampleRate = int(engine.get_samplerate());
+    assert(sampleRate>0);
+    
+    switch(latency) {
+      case(0):
+        if (numSampsToProcess <= 2048) {
+          fftFrameSize = 512 ; 
+        } else {
+          fftFrameSize = numSampsToProcess*0.25 ;
+        }
+        break;
+      case(1):
+        fftFrameSize = numSampsToProcess;
+        break;
+      case(2):
+        fftFrameSize = numSampsToProcess*0.25;
+        break;
+      default:
+        if (numSampsToProcess <= 2048) {
+          fftFrameSize = 512 ; 
+        } else {
+          fftFrameSize = numSampsToProcess*0.25 ;
+        }
+        break;
+    }
+    fftFrameSize2 = fftFrameSize/2;
+
+    try {
+        //create FFTW plan
+        ftPlanForward = fftwf_plan_dft_1d(fftFrameSize, fftw_in, fftw_out, FFTW_FORWARD, FFTW_ESTIMATE);
+        ftPlanInverse = fftwf_plan_dft_1d(fftFrameSize, fftw_in, fftw_out, FFTW_BACKWARD, FFTW_ESTIMATE);
+        // alloc buffers
+        fpb = new float[fftFrameSize2];
+        expect = new float[fftFrameSize2];
+        hanning = new float[fftFrameSize];
+        hanningd = new float[fftFrameSize];
+        resampin = new float[fftFrameSize];
+        resampin2 = new float[fftFrameSize];
+        resampout = new float[fftFrameSize*4];
+        indata2 = new float[fftFrameSize*4];
+    } catch(...) {
+            gx_print_error("detune", "cant allocate memory pool");
+            return;
+        }
+    clear_state();
+}
+
+void smbPitchShift::mem_free()
+{
+    ready = false;
+    mem_allocated = false;
+    if (fpb) { delete fpb; fpb = 0; }
+    if (expect) { delete expect; expect = 0; }
+    if (hanning) { delete hanning; hanning = 0; }
+    if (hanningd) { delete hanningd; hanningd = 0; }
+    if (resampin) { delete resampin; resampin = 0; }
+    if (resampin2) { delete resampin2; resampin2 = 0; }
+    if (resampout) { delete resampout; resampout = 0; }
+    if (indata2) { delete indata2; indata2 = 0; }
+    if (ftPlanForward)
+        {fftwf_destroy_plan(ftPlanForward);ftPlanForward = 0; }
+    if (ftPlanInverse) 
+        { fftwf_destroy_plan(ftPlanInverse);ftPlanInverse = 0; }
+}
+
+int smbPitchShift::activate(bool start)
+{
+    if (start) {
+        if (!mem_allocated) {
+            mem_alloc();
+        }
+    } else if (mem_allocated) {
+        mem_free();
+    }
+    return 0;
+}
+
+void smbPitchShift::change_buffersize(unsigned int size)
+{
+    sync();
+    ready = false;
+    if (mem_allocated) {
+        mem_free();
+        mem_alloc();
+    }
+}
+
+void smbPitchShift::change_latency()
+{
+    sync();
+    ready = false;
+    if (mem_allocated) {
+        mem_free();
+        mem_alloc();
+    }
+}
+
+smbPitchShift::~smbPitchShift()
+{
+    if (fpb) { delete fpb; fpb = 0; }
+    if (expect) { delete expect; expect = 0; }
+    if (hanning) { delete hanning; hanning = 0; }
+    if (hanningd) { delete hanningd; hanningd = 0; }
+    if (resampin) { delete resampin; resampin = 0; }
+    if (resampin2) { delete resampin2; resampin2 = 0; }
+    if (resampout) { delete resampout; resampout = 0; }
+    if (indata2) { delete indata2; indata2 = 0; }
+    if (ftPlanForward)
+        {fftwf_destroy_plan(ftPlanForward);ftPlanForward = 0; }
+    if (ftPlanInverse)
+        {fftwf_destroy_plan(ftPlanInverse);ftPlanInverse = 0; }
+}
+
+// -----------------------------------------------------------------------------------------------------------------
+void __rt_func smbPitchShift::compute_static(int count, float *input0, float *output0, PluginDef *p)
+{
+    static_cast<smbPitchShift*>(p)->PitchShift(count, input0, output0);
+}
+
+
+void always_inline smbPitchShift::PitchShift(int count, float *indata, float *outdata)
+{
+    
+    if (!ready || count != numSampsToProcess)  {
+        memcpy(outdata,indata,count*sizeof(float));
+        return;
+    }
+    
+    resamp.down(numSampsToResamp,indata,resampin);
+    double     fSlow0 = (0.01 * wet);
+    double     fSlow1 = (0.01 * dry);
+
+    // collect data for latency compensation
+    for (i = 0; i < count; i++){
+        indata2[ii] = indata[i];
+        ii++;
+    }
+    // collect data for fft
+    for (i = 0; i < numSampsToResamp; i++){
+        resampin2[ai] = resampin[i];
+        ai++;
+    }
+    // now we have enough data
+    if (ai>=fftFrameSize) {
+        ai = 0;
+        ii = 0;
+        switch(octave) {
+          case(0):
+            tone =0;
+            break;
+          case(1):
+            tone =12;
+            break;
+          case(2):
+            tone =-12;
+            break;
+          default:
+            tone =0;
+            break;
+        }
+        float pitchShift = pow(2., (semitones+tone)*0.0833333333);
+        /* main processing loop */
+        for (i = 0; i < fftFrameSize; i++){
+
+            /* As long as we have not yet collected enough data just read in */
+            float fTemp = resampin2[i];
+            gInFIFO[gRover] = fTemp;
+            resampin2[i] = gOutFIFO[gRover-inFifoLatency];
+            gRover++;
+
+            /* now we have enough data for processing */
+            if (gRover >= fftFrameSize) {
+                gRover = inFifoLatency;
+
+                /* do windowing and re,im interleave */
+                for (k = 0; k < fftFrameSize;k++) {
+                    fftw_in[k][0] = gInFIFO[k] * hanning[k];
+                    fftw_in[k][1] = 0.0;
+                }
+
+
+                /* ***************** ANALYSIS ******************* */
+                /* do transform */
+                fftwf_execute(ftPlanForward);
+
+                /* this is the analysis step */
+                for (k = 0; k < fftFrameSize2; k++) {
+
+                    /* de-interlace FFT buffer */
+                    real = fftw_out[k][0];
+                    imag = fftw_out[k][1];
+
+                    /* compute magnitude and phase */
+                    magn = 2.*sqrt(real*real + imag*imag);
+                    phase = atan2(imag,real);
+
+                    /* compute phase difference */
+                    tmp = phase - gLastPhase[k];
+                    gLastPhase[k] = phase;
+
+                    /* subtract expected phase difference */
+                    tmp -= expect[k];
+
+                    /* map delta phase into +/- Pi interval */
+                    qpd = tmp*mpi1;
+                    if (qpd >= 0) qpd += qpd&1;
+                    else qpd -= qpd&1;
+                    tmp -= M_PI*(double)qpd;
+
+                    /* get deviation from bin frequency from the +/- Pi interval */
+                    /* compute the k-th partials' true frequency */
+                    tmp = fpb[k] + tmp*freqPerBin2;
+
+                    /* store magnitude and true frequency in analysis arrays */
+                    gAnaMagn[k] = magn;
+                    gAnaFreq[k] = tmp;
+
+                }
+
+                /* ***************** PROCESSING ******************* */
+                /* this does the actual pitch shifting */
+                memset(gSynMagn, 0, fftFrameSize*sizeof(float));
+                memset(gSynFreq, 0, fftFrameSize*sizeof(float));
+                for (k = 1; k < fftFrameSize2-2; k++) { 
+                    index = k*pitchShift;
+                    if (index < fftFrameSize2) { 
+                        if (index < fftFrameSize2*0.20)
+                            gSynMagn[index] += gAnaMagn[k]*a; 
+                        else if (index < fftFrameSize2*0.45)
+                            gSynMagn[index] += gAnaMagn[k]*b; 
+                        else if (index < fftFrameSize2*0.667)
+                            gSynMagn[index] += gAnaMagn[k]*c; 
+                        else 
+                            gSynMagn[index] += gAnaMagn[k]*d; 
+                           gSynFreq[index] = gAnaFreq[k] * pitchShift; 
+                    } 
+                }
+                
+                /* ***************** SYNTHESIS ******************* */
+                /* this is the synthesis step */
+                for (k = 0; k < fftFrameSize2; k++) {
+
+                    /* get magnitude and true frequency from synthesis arrays */
+                    magn = gSynMagn[k];
+                    //tmp = gSynFreq[k];
+
+                    /* subtract bin mid frequency */
+                    /* get bin deviation from freq deviation */
+                    /* take osamp into account */
+                    /* add the overlap phase advance back in */
+                    tmp = ((gSynFreq[k] - fpb[k]) * freqPerBin1) + expect[k];
+
+                    /* accumulate delta phase to get bin phase */
+                    gSumPhase[k] += tmp;
+                    phase = gSumPhase[k];
+                    if (magn == 0.0) continue;
+
+                    /* get real and imag part and re-interleave */                
+                    fftw_in[k][0] = magn * cos (phase);
+                    fftw_in[k][1] = magn * sin (phase);
+                } 
+
+                /* do inverse transform */
+                fftwf_execute(ftPlanInverse);
+                /* do windowing and add to output accumulator */ 
+                for(k=0; k < fftFrameSize; k++) {
+                    gOutputAccum[k] += hanningd[k] * fftw_out[ k][0] ;
+                }
+                for (k = 0; k < stepSize; k++) gOutFIFO[k] = gOutputAccum[k];
+
+                /* shift accumulator */
+                memmove(gOutputAccum, gOutputAccum+stepSize, fftFrameSize*sizeof(float));
+
+                /* move input FIFO */
+                for (k = 0; k < inFifoLatency; k++) gInFIFO[k] = gInFIFO[k+stepSize];
+            }
+        }
+        resamp.up(fftFrameSize,resampin2,resampout);
+        aio = 0;
+    }
+    if(l) {
+        for (i = 0; i < count; i++){
+            outdata[i] = ((fSlow0 * resampout[aio]) + (fSlow1 *indata2[aio]));
+            aio++;
+        }
+    } else {
+        for (i = 0; i < count; i++){
+            outdata[i] = ((fSlow0 * resampout[aio]) + (fSlow1 *indata[i]));
+            aio++;
+        }
+    }
+}
+
+int smbPitchShift::register_par(const ParamReg& reg) 
+{
+    reg.registerVar("smbPitchShift.semitone", N_("detune"), "S", "", &semitones, 0.0, -0.25, 0.25, 0.01);
+    static const value_pair octave_values[] = {{"unison"},{"octave up"},{"octave down"},{0}};
+    reg.registerIEnumVar("smbPitchShift.octave",N_("add harmonics"),"B",N_("add harmonics"),octave_values,&octave, 0);
+    static const value_pair latency_values[] = {{"latency "},{"compensate"},{0}};
+    reg.registerEnumVar("smbPitchShift.l",N_("compensate latency"),"S",N_("compensate latency"),latency_values,&l, 0.0f, 0.0f, 1.0f, 1.0f);
+    static const value_pair latency_set[] = {{"high quality"},{"low quality"},{"realtime"},{0}};
+    reg.registerIEnumVar("smbPitchShift.latency",N_("latency settings"),"B",N_("latency settings"),latency_set,&latency, 0);
+    reg.registerVar("smbPitchShift.wet", N_("wet amount"), "S", "", &wet, 50.0, 0.0, 100.0, 1);
+    reg.registerVar("smbPitchShift.dry", N_("dry amount"), "S", "", &dry, 50.0, 0.0, 100.0, 1);
+    reg.registerVar("smbPitchShift.a", N_("low"), "S", N_("low"), &a, 1.0, 0.0, 2.0, 0.01);
+    reg.registerVar("smbPitchShift.b", N_("middle low"), "S", N_("middle low"), &b, 1.0, 0.0, 2.0, 0.01);
+    reg.registerVar("smbPitchShift.c", N_("middle treble"), "S", N_("middle treble"), &c, 1.0, 0.0, 2.0, 0.01);
+    reg.registerVar("smbPitchShift.d", N_("treble"), "S", N_("treble"), &d, 1.0, 0.0, 2.0, 0.01);
+    param["smbPitchShift.latency"].signal_changed_int().connect(
+        sigc::hide(sigc::mem_fun(this, &smbPitchShift::change_latency)));
+    return 0;
+}
+
+int smbPitchShift::registerparam(const ParamReg& reg) 
+{
+    return static_cast<smbPitchShift*>(reg.plugin)->register_par(reg);
+}
+
+int smbPitchShift::load_ui_f(const UiBuilder& b, int form) 
+{
+    if (form & UI_FORM_STACK) {
+    b.openHorizontalhideBox("");
+    {
+        b.create_master_slider("smbPitchShift.semitone",0);
+    }
+    b.closeBox();
+    b.openVerticalBox("");
+    {
+    b.openHorizontalBox("");
+    {
+    b.openVerticalBox("");
+    {
+    b.create_selector_no_caption("smbPitchShift.octave");
+    b.create_selector_no_caption("smbPitchShift.l");
+    b.create_selector_no_caption("smbPitchShift.latency");
+    }
+    b.closeBox();
+    b.create_small_rackknob("smbPitchShift.semitone",0);
+    b.create_small_rackknob("smbPitchShift.dry",0);
+    b.create_small_rackknob("smbPitchShift.wet",0);
+    }
+    b.closeBox();
+    b.openHorizontalBox("");
+    {
+    b.create_small_rackknob("smbPitchShift.a",0);
+    b.create_small_rackknob("smbPitchShift.b",0);
+    b.create_small_rackknob("smbPitchShift.c",0);
+    b.create_small_rackknob("smbPitchShift.d",0);
+    }
+    b.closeBox();
+    }
+    b.closeBox();
+    return 0;
+    }
+    return -1;
+}
+
+int smbPitchShift::activate_static(bool start, PluginDef *p)
+{
+    return static_cast<smbPitchShift*>(p)->activate(start);
+}
+
+int smbPitchShift::load_ui_f_static(const UiBuilder& b, int form)
+{
+    return static_cast<smbPitchShift*>(b.plugin)->load_ui_f(b, form);
+}
+
+void smbPitchShift::del_instance(PluginDef *p)
+{
+    delete static_cast<smbPitchShift*>(p);
+}
+
+
 } // namespace gx_engine
