@@ -910,6 +910,13 @@ void PresetWindow::on_preset_edited(const Glib::ustring& path, const Glib::ustri
 	reset_edit(*preset_treeview->get_column(0));
 	return;
     }
+    // vowel mutation lead to a Glib::markup error when building the menu action,
+    // so check for it and avoid them
+    if (newname.find_first_of("ÜüÖöÄä") != Glib::ustring::npos) {
+	gx_print_error("PresetWindow", "Vowel mutation isn't allowed in preset names, please choose a name without (ÜüÖöÄä)");
+    reset_edit(*preset_treeview->get_column(0));
+	return;        
+    }
     gx_system::PresetFileGui& fl = *machine.get_bank_file(get_current_bank());
     Glib::ustring t = newname;
     int n = 1;
