@@ -78,7 +78,7 @@ PluginDisplay::PluginDisplay(gx_engine::GxMachineBase& machine_, Glib::RefPtr<Gd
       on_reordered_conn(), display_type_list(), display_type_list_sr(), output_type_list(),
       finished_callback(finished_callback_)
 {
-    std::vector<unsigned long> old_not_found;
+    std::vector<std::string> old_not_found;
     machine.load_ladspalist(old_not_found, pluginlist);
     bld = gx_gui::GxBuilder::create_from_file(machine.get_options().get_builder_filepath("ladspaliste.glade"));
     bld->get_toplevel("window1", window);
@@ -92,6 +92,8 @@ PluginDisplay::PluginDisplay(gx_engine::GxMachineBase& machine_, Glib::RefPtr<Gd
     bld->find_widget("combobox_mono_stereo", combobox_mono_stereo);
     bld->find_widget("selected_only", selected_only);
     bld->find_widget("changed_only", changed_only);
+    bld->find_widget("ladspa_only", ladspa_only);
+    bld->find_widget("lv2_only", lv2_only);
     bld->find_widget("show_all", show_all);
     bld->find_widget("details_box", details_box);
     bld->find_widget("plugin_name", plugin_name);
@@ -253,6 +255,8 @@ PluginDisplay::PluginDisplay(gx_engine::GxMachineBase& machine_, Glib::RefPtr<Gd
 
     selected_only->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &PluginDisplay::on_view_changed), selected_only));
     changed_only->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &PluginDisplay::on_view_changed), changed_only));
+    ladspa_only->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &PluginDisplay::on_view_changed), ladspa_only));
+    lv2_only->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &PluginDisplay::on_view_changed), lv2_only));
     show_all->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &PluginDisplay::on_view_changed), show_all));
 
     bld->find_widget("combobox_mono_stereo", cb);
@@ -290,7 +294,7 @@ static void split(std::vector<ustring>& strvec, const ustring& str) {
         size_t end = str.find_first_of(' ', start);
 	size_t len = (end == np) ? np : end - start;
 	if (len > 0) {
-	    strvec.push_back(str.substr(start, len));
+	    strvec.push_back(str.substr(start, len).lowercase());
 	}
 	if (end == np) {
 	    return;
@@ -558,17 +562,26 @@ void PluginDisplay::load() {
     int a = combobox_mono_stereo->get_model()->get_path(combobox_mono_stereo->get_active())[0];
     bool s = selected_only->get_active();
     bool c = changed_only->get_active();
+    bool d = ladspa_only->get_active();
+    bool e = lv2_only->get_active();
     plugin_liststore->clear();
     for (std::vector<PluginDesc*>::iterator v = pluginlist.begin(); v != pluginlist.end(); ++v) {
 	if (s && !(*v)->active) {
 	    continue;
 	}
-	if (c && !(*v)->has_settings) {
+	else if (c && !(*v)->has_settings) {
+	    continue;
+	}
+	else if (d && (*v)->is_lv2) {
+	    continue;
+	}
+ 	else if (e && !(*v)->is_lv2) {
 	    continue;
 	}
 	if ((a == 1 && (*v)->tp != 0) || (a == 2 && (*v)->tp != 1)) {
 	    continue;
 	}
+
 	Gtk::TreeIter it = plugin_liststore->append();
 	it->set_value(plugin_liststore->col.name, ustring((*v)->Name));
 	it->set_value(plugin_liststore->col.active, (*v)->active);
@@ -680,7 +693,7 @@ void PluginDisplay::save_current() {
 }
 
 void PluginDisplay::set_title() {
-    ustring t = _("Select LADSPA plugins for Guitarix");
+    ustring t = _("Select LADSPA/LV2 plugins for Guitarix");
     if (change_count) {
 	window->set_title(t + _(" (*changed*)"));
     } else {
@@ -768,7 +781,11 @@ void PluginDisplay::selection_changed() {
     dry_wet_button->set_active(current_plugin->add_wet_dry);
     ladspa_category->set_text(p->ladspa_category);
     ladspa_maker->set_text(p->Maker);
-    ladspa_uniqueid->set_text(ustring::compose("%1: %2[%3]", p->UniqueID, p->path, p->index));
+    if (p->is_lv2) {
+	ladspa_uniqueid->set_text(ustring::compose("LV2: %1", p->path));
+    } else {
+	ladspa_uniqueid->set_text(ustring::compose("%1: %2[%3]", p->UniqueID, p->path, p->index));
+    }
     for (unsigned int i = 0; i < p->ctrl_ports.size(); ++i) {
 	PortDesc *q = p->ctrl_ports[i];
 	Glib::RefPtr<Gtk::ListStore> tls;
