@@ -655,7 +655,14 @@ void PluginPresetPopup::save_plugin_preset() {
     InputWindow *w = InputWindow::create(machine.get_options(), save_name_default);
     w->run();
     if (!w->get_name().empty()) {
-	machine.plugin_preset_list_save(pdef, w->get_name());
+        // save loop file to plugin preset name
+	    if(strcmp(pdef->id,"dubber")==0) {
+            Glib::ustring name = "";
+            machine.set_parameter_value("dubber.filename", name);
+            machine.set_parameter_value("dubber.savefile", true);
+            machine.set_parameter_value("dubber.filename", w->get_name());
+        }
+        machine.plugin_preset_list_save(pdef, w->get_name());
     }
     delete w;
 }
