@@ -22,6 +22,8 @@
 
 #pragma once
 
+#include <lilv/lilv.h>
+
 namespace gx_jack { class GxJack; }
 
 namespace gx_engine {
@@ -534,7 +536,7 @@ struct paradesc: boost::noncopyable {
     void writeJSON(gx_system::JsonWriter& jw);
 };
 
-enum quirkflag { need_activate = 1, no_cleanup = 2 };
+enum quirkflag { need_activate = 1, no_cleanup = 2, is_lv2 = 4 };
 
 class plugdesc {
 public:
@@ -565,6 +567,12 @@ public:
 private:
     const gx_system::CmdlineOptions& options;
     pluginarray plugins;
+    LilvWorld* world;
+    const LilvPlugins* lv2_plugins;
+    LilvNode* lv2_AudioPort;
+    LilvNode* lv2_ControlPort;
+    LilvNode* lv2_InputPort;
+    LilvNode* lv2_OutputPort;
 private:
     void read_module_config(const std::string& filename, plugdesc *p);
     void read_module_list(pluginarray& p);
@@ -577,11 +585,14 @@ public:
     PluginDef *create(const plugdesc *p);
     pluginarray::iterator begin() { return plugins.begin(); }
     pluginarray::iterator end() { return plugins.end(); }
-    pluginarray::iterator find(unsigned long uniqueid);
+    pluginarray::iterator find(plugdesc* desc);
     void set_plugins(pluginarray& new_plugins);
     void update_instance(PluginDef *pdef, plugdesc *pdesc);
     static std::string get_ladspa_filename(unsigned long uid)
 	{ return "ladspa"+gx_system::to_string(uid)+".js"; }
+    static std::string get_ladspa_filename(std::string uid_key)
+	{ return "ladspa"+uid_key.substr(9)+".js"; }
+    friend class Lv2Dsp;
 };
 
 
@@ -785,4 +796,102 @@ public:
 };
 
 
+/****************************************************************************
+*
+* NAME: smbPitchShift.cpp
+* VERSION: 1.2
+* HOME URL: http://www.dspdimension.com
+* KNOWN BUGS: none
+* 
+*
+* COPYRIGHT 1999-2009 Stephan M. Bernsee <smb [AT] dspdimension [DOT] com>
+* 
+* Modified for guitarix by Hermann Meyer 2014
+*
+* 						The Wide Open License (WOL)
+*
+* Permission to use, copy, modify, distribute and sell this software and its
+* documentation for any purpose is hereby granted without fee, provided that
+* the above copyright notice and this license appear in all source copies. 
+* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF
+* ANY KIND. See http://www.dspguru.com/wol.htm for more information.
+*
+*****************************************************************************/ 
+
+
+#define M_PI 3.14159265358979323846
+#define MAX_FRAME_LENGTH 8096
+
+class smbPitchShift : public PluginDef {
+private:
+    gx_resample::SimpleResampler resamp;
+    EngineControl&  engine;
+	bool            mem_allocated;
+    sigc::slot<void> sync;
+	volatile bool ready;
+    ParamMap& param;
+	float gInFIFO[MAX_FRAME_LENGTH];
+	float gOutFIFO[MAX_FRAME_LENGTH];
+    float *fpb;
+    float *expect;
+    float *hanning;
+    float *hanningd;
+    float *resampin;
+    float *resampin2;
+    float *resampout;
+    float *indata2;
+	float gLastPhase[MAX_FRAME_LENGTH/2+1];
+	float gSumPhase[MAX_FRAME_LENGTH/2+1];
+	float gOutputAccum[2*MAX_FRAME_LENGTH];
+	float gAnaFreq[MAX_FRAME_LENGTH];
+	float gAnaMagn[MAX_FRAME_LENGTH];
+	float gSynFreq[MAX_FRAME_LENGTH];
+	float gSynMagn[MAX_FRAME_LENGTH];
+	float semitones;
+	float a,b,c,d,l;
+	float wet;
+	float dry;
+    float mpi, mpi1;
+    float tone;
+	int   octave, osamp, numSampsToResamp, numSampsToProcess, fftFrameSize, sampleRate ;
+	int latency;
+    int ai;
+    int aio;
+    int ii;
+	long  gRover , gInit ;
+	double magn, phase, tmp, window, real, imag;
+	double freqPerBin, freqPerBin1, freqPerBin2, expct;
+    double fftFrameSize3;
+    double fftFrameSize4;
+    double osamp1,osamp2;
+	long   i,k, qpd, index, inFifoLatency, stepSize, fftFrameSize2;
+	
+    fftwf_complex fftw_in[MAX_FRAME_LENGTH], fftw_out[MAX_FRAME_LENGTH];
+    fftwf_plan ftPlanForward, ftPlanInverse;
+    
+    inline int load_ui_f(const UiBuilder& b, int form);
+	int register_par(const ParamReg& reg);
+    void change_latency();
+   
+    void mem_alloc();
+	void mem_free();
+    void clear_state();
+	int activate(bool start);
+	bool setParameters( int sampleRate);
+	void PitchShift(int count, float *indata, float *outdata);
+    void change_buffersize(unsigned int size);
+    static int  activate_static(bool start, PluginDef*);
+    static void del_instance(PluginDef *p);
+    static int registerparam(const ParamReg& reg);
+    static int load_ui_f_static(const UiBuilder& b, int form);
+	static void init(unsigned int sampleRate, PluginDef *plugin); 
+    static void compute_static(int count, float *input0, float *output0, PluginDef *p); 
+
+public:
+    Plugin plugin;
+	smbPitchShift(ParamMap& param_, EngineControl& engine, sigc::slot<void> sync);
+	~smbPitchShift();
+};
+
+
 } // namespace gx_engine
