@@ -748,6 +748,7 @@ public:
     ContrastConvolver contrast;
     LiveLooper  loop;
     SCapture record;
+    smbPitchShift detune;
 public:
     MonoEngine(const string& plugin_dir, const string& loop_dir, ParameterGroups& groups);
     ~MonoEngine();
@@ -912,7 +913,7 @@ MonoEngine::MonoEngine(const string& plugin_dir, const string& loop_dir, Paramet
       preamp(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       loop(get_param(), sigc::mem_fun(mono_chain, &MonoModuleChain::sync), loop_dir),
-      record(*this, 1) {
+      record(*this, 1), detune(get_param(), *this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync)) {
 
     mono_convolver.set_sync(true);
     cabinet.set_sync(true);
@@ -992,6 +993,7 @@ void MonoEngine::load_static_plugins() {
     pl.add(&crybaby.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&loop.plugin,                          PLUGIN_POS_RACK,  PGN_GUI);
     pl.add(&record.plugin,                      PLUGIN_POS_RACK,  PGN_GUI);
+    pl.add(&detune.plugin,                      PLUGIN_POS_RACK,  PGN_GUI);
     pl.add(gx_effects::gx_distortion::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::ts9sim::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::impulseresponse::plugin(), PLUGIN_POS_RACK, PGN_GUI);
@@ -1013,6 +1015,7 @@ void MonoEngine::load_static_plugins() {
     pl.add(gx_effects::ring_modulator::plugin(),  PLUGIN_POS_RACK, PGN_GUI);
 	pl.add(gx_effects::duck_delay::plugin(),      PLUGIN_POS_RACK);
 	pl.add(pluginlib::reversedelay::plugin(),     PLUGIN_POS_RACK);
+	pl.add(gx_effects::baxandall::plugin(),      PLUGIN_POS_RACK);
     pl.add(&tonestack.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&mono_convolver.plugin,                PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&cabinet.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
