@@ -1,42 +1,12 @@
 # -*- coding: utf-8 -*-
 from __future__ import division
 
+import math
 import pylab as pl
 import numpy as np
 from models import *
-
-def genlogsweep(fmin, fmax, rate, k0, k1, k2, dtype=np.float64):
-    """generate logarithmic sweep signal
-    
-    fmin: start frequency
-    fmax: end frequency
-    rate: sample rate
-    k0: fade in before start of signal (samples)
-    k1: length of signal (samples)
-    k2: fade out after end of signal (samples)
-    dtype: data type of signal
-
-    returns: (logsweep signal, inverse logsweep signal)
-    """
-    s1 = np.empty(k0 + k1 + k2, dtype=dtype)
-    s2 = np.empty_like(s1)
-    b = math.log(fmax / fmin) / k1
-    a = fmin / (b * rate)
-    r = 0.5 * a * (fmax / fmin) * (k1 + 0.5 * (k0 + k2)) / (b * k1)
-    q0 = a * math.exp(-b * k0)
-    for i in range(-k0, k1+k2):
-        if i < 0:
-            g = math.cos(0.5 * math.pi * i / k0)
-        elif i < k1:
-            g = 1.0
-        else:
-            g = math.sin(0.5 * math.pi * (k1 + k2 - i) / k2)
-        q = a * math.exp(b * i)
-        p = q - q0
-        x = g * math.sin(2 * math.pi * (p - math.floor(p)))
-        s1[k0+i] = x
-        s2[-(k0+i+1)] = x * q / r
-    return s1, s2
+import dk_simulator, dk_lib
+from numpy.fft import fftfreq
 
 class Test(object):
     FS = 96000
@@ -51,12 +21,35 @@ class Test(object):
     def compare_data(self, data):
         error = np.max(abs(self.result - self.get_samples(data))) / np.max(abs(data))
         if (error > self.max_error).any():
-            return "Difference = %g (> %g)" % (error, self.max_error)
+            return False, "Difference = %g (> %g)" % (error, self.max_error)
         else:
-            return "OK"
+            return True, "OK"
 
-    def print_data(self, data):
-        print repr(self.get_samples(data))
+    def check_signal(self, p):
+        try:
+            y = p(self.signal())
+        except ValueError as v:
+            return False, v
+        else:
+            return self.compare_data(y)
+
+    def check(self, name, args):
+        parser = dk_simulator.Parser(self.S, self.V, self.FS, not args.backward_euler)
+        p = dk_simulator.get_executor(
+            name, parser, self.solver, args.pure_python, c_tempdir=args.c_tempdir,
+            c_verbose=args.c_verbose, c_debug_load=args.c_debug_load, linearize=args.linearize)
+        if args.print_result:
+            self.print_data(p(self.signal()), "\nresult = np.")
+            return ""
+        res = self.check_signal(p)  # side-effect: calculate time_per_sample
+        if hasattr(p, "time_per_sample"):
+            s = " [%.2g]" % p.time_per_sample
+        else:
+            s = ""
+        return "%s%s" % (res[1], s)
+
+    def print_data(self, data, prefix=""):
+        print prefix + repr(self.get_samples(data))
 
     def op_signal(self, op=None, samples=None, timespan=None):
         if timespan is None:
@@ -92,10 +85,50 @@ class Test(object):
         pl.legend(args, loc=loc)
         pl.show()
 
+    def impulse(self, p, magnitude=1e-3):
+        a = self.op_signal(samples=64*1024, op=self.V.get("OP",[0.]))
+        a[0,0] += magnitude
+        return (p(a)-p.o0) / magnitude
+
+    def make_sweep(self, pre=None, span=0.5, post=0.1, magnitude=1e-2, start=20, stop=10000):
+        smpl = lambda tm: int(round(tm*self.FS))
+        if pre is None:
+            pre = span/2
+        s, d = dk_lib.genlogsweep(
+            start, stop, self.FS, smpl(pre), smpl(span), smpl(post))
+        s *= magnitude
+        n = dk_lib.pow2roundup(len(s))
+        #d /= np.mean(abs(np.fft.fft(dk_lib.fft_convolve(d, s), n))[n*start/self.FS:n*stop/self.FS])
+        #return s, d
+        return s
+
+    def sweep(self, p, pre=None, span=0.5, post=0.1, magnitude=1e-2, start=20, stop=10000):
+        #s, d = self.make_sweep(pre, span, post, magnitude, start, stop)
+        s = self.make_sweep(pre, span, post, magnitude, start, stop)
+        a = self.op_signal(samples=len(s), op=self.V.get("OP",[0.]))
+        a[:,0] += s
+        y = p(a)-p.o0
+        #return dk_lib.fft_convolve(d, y[:,0])
+        return dk_lib.fft_convolve(s, y[:,0], invert=True)
+
+    #spectrum_signal = impulse
+    spectrum_signal = sweep
+
+    def plot_spectrum(self, p, plot_variable):
+        y = self.spectrum_signal(p)
+        n = dk_lib.pow2roundup(len(y))
+        cut = slice(n*20.0/self.FS, n*10000.0/self.FS)
+        w = fftfreq(n,1.0/self.FS)[cut]
+        def spec(y):
+            s = 20*np.log10(abs(np.fft.fft(y,n,axis=0)[cut]))
+            return np.where(s > -80, s, np.nan)
+        pl.semilogx(w, spec(y))
+        self.finish_plot(p.out_labels)
+
 
 class Pot_test(Test):
 
-    S = ((P(), 1, 2, GND),
+    S = ((P(), GND, 1, 2),
          (IN, 1),
          (OUT, 2),
          )
@@ -109,10 +142,11 @@ class Pot_test(Test):
     def signal(self):
         return self.constant_signal(10)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels)
+        self.finish_plot(p.out_labels)
 
 
 class Choke_test(Test):
@@ -127,7 +161,7 @@ class Choke_test(Test):
     V = {R(): 100,
          Trans_L(): dict(windings=[100], R = 358.),
          Trans_F(): dict(windings=[100], b = 255., c = 358.),
-         Trans_GC(): dict(windings=[100], C = 3e-3, a = 5, n = 7),
+         Trans_GC(): dict(windings=[100], C = 3e-3, a = 5, o = 0, n = 7),
          "OP": [0],
          }
 
@@ -147,10 +181,11 @@ class Choke_test(Test):
     def signal(self):
         return self.constant_signal(10)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
 
 
 class Transformer_GC_test(Test): # transformer
@@ -168,7 +203,7 @@ class Transformer_GC_test(Test): # transformer
     V = {R("p1"): 100,
          R("p2"): 100,
          R("s"): 1e3,
-         Trans_GC(): dict(windings=[100, 100, 100], C=2e-3, a=1e-5, n=25),
+         Trans_GC(): dict(windings=[100, 100, 100], C=2e-3, a=1e-5, o=0, n=25),
          Trans_F(): dict(windings=[100, 100, 100], b = 255., c = 358.),
          Trans_L(): dict(windings=[100, 100, 100], R = 358.),
          }
@@ -185,7 +220,6 @@ class Transformer_GC_test(Test): # transformer
         [-0.3746337 ,  1.22787577,  0.39689002],
         [-0.36713395, -0.19608725,  0.19608725]])
 
-    solver = dict(method='hybr')
     freq = 40.
     timespan = 2 / freq
 
@@ -194,10 +228,27 @@ class Transformer_GC_test(Test): # transformer
         a[:,1] = -a[:,1]
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
+
+    def spectrum_signal(self, p):
+        if 0:
+            s, d = self.make_sweep()
+            a = self.op_signal(samples=len(s), op=self.V.get("OP",[0.,0.]))
+            a[:,0] += s
+            a[:,1] -= s
+            y = p(a)-p.o0
+            return dk_lib.fft_convolve(d, y)
+        else:
+            magnitude = 1e-3
+            a = self.op_signal(samples=64*1024, op=self.V.get("OP",[0.,0.]))
+            a[0,0] += magnitude
+            a[0,1] -= magnitude
+            return (p(a)-p.o0) / magnitude
+        
 
 class PushPullTransformer_test(Test): # 2 push-pull pentodes with transformer
     #Trans_ = Trans_F
@@ -232,37 +283,36 @@ class PushPullTransformer_test(Test): # 2 push-pull pentodes with transformer
          R(1): 100,
          R(2): 100,
          Trans_L(nw=3): dict(windings=[1000, 1000, 64], R=358),
-         Trans_GC(nw=3): dict(windings=[100.0, 100.0, 6.4], C=24e-3, a=900, n=7),
+         Trans_GC(nw=3): dict(windings=[100.0, 100.0, 6.4], C=24e-3, a=900, o=0, n=7),
          #Trans_GC(nw=3): dict(windings=[1000, 1000, 64], C=2e-3, a=1e-5, n=25),
          Trans_F(nw=3): dict(windings=[1000, 1000, 64], b = 255., c = 358.),
          "Vps": 394,
          "OP": [-37., -37., 394.],
          }
 
-    result = np.array([[  8.85087837e-11,   7.12940817e-12,   5.26565523e-02,
-         -5.26565523e-02],
-       [ -1.58278792e+00,   1.14670412e-01,   5.00849572e-01,
-         -4.28188725e-05],
-       [ -1.38833333e+00,  -7.39092672e-01,   1.69924724e-01,
-         -6.89825386e-03],
-       [  5.73079885e-01,  -3.83696969e+00,   1.14213877e-04,
-         -1.99353250e-01],
-       [  1.49571385e+00,   4.47570165e-01,   6.88773238e-04,
-         -3.12632460e-01],
-       [  1.32753760e-01,   3.77081469e+00,   1.93209752e-01,
-         -6.97194440e-04],
-       [ -1.55895858e+00,  -2.42708781e-01,   4.33324817e-01,
-         -1.12964367e-04],
-       [ -7.93509924e-01,  -3.08007847e+00,   6.98993965e-03,
-         -1.57496524e-01],
-       [  1.58649719e+00,   3.56694151e-02,   4.27872401e-05,
-         -5.01235149e-01],
-       [  1.19796324e+00,   1.43745392e+00,   5.20594452e-02,
-         -5.29346715e-02]])
+    result = np.array([[  6.65876850e-14,  -5.33887551e-16,   5.15526850e-02,
+         -5.15526850e-02],
+       [ -1.56810408e+00,   1.14280615e-01,   4.69924498e-01,
+         -4.28187935e-05],
+       [ -1.37921862e+00,  -7.20797506e-01,   1.62033720e-01,
+         -6.89921104e-03],
+       [  5.51916398e-01,  -3.82772153e+00,   1.14212528e-04,
+         -1.98699316e-01],
+       [  1.48378173e+00,   4.35317909e-01,   6.88827024e-04,
+         -2.95292058e-01],
+       [  1.51544857e-01,   3.75136569e+00,   1.92135627e-01,
+         -6.97176803e-04],
+       [ -1.54529256e+00,  -2.35928614e-01,   4.07366368e-01,
+         -1.12968725e-04],
+       [ -8.02005162e-01,  -2.97468382e+00,   6.98876783e-03,
+         -1.51762917e-01],
+       [  1.57166756e+00,   2.26646240e-02,   4.27900722e-05,
+         -4.70255081e-01],
+       [  1.19318722e+00,   1.40055656e+00,   5.09913605e-02,
+         -5.18198164e-02]])
 
     freq = 30.
     timespan = 2 / freq
-    solver = dict(method='hybr')
 
     def signal(self):
         a = 50*self.sine_signal(self.freq, 3)
@@ -271,19 +321,77 @@ class PushPullTransformer_test(Test): # 2 push-pull pentodes with transformer
         a[:,2] = self.V["OP"][2]
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
+
+    def spectrum_signal(self, p):
+        if 0:
+            s, d = self.make_sweep()
+            a = self.op_signal(samples=len(s), op=self.V["OP"])
+            a[:,0] += s
+            a[:,1] -= s
+            y = p(a)-p.o0
+            return dk_lib.fft_convolve(d, y)
+        else:
+            magnitude = 1e-3
+            a = self.op_signal(samples=64*1024, op=self.V["OP"])
+            a[0,0] += magnitude
+            a[0,1] -= magnitude
+            return (p(a)-p.o0) / magnitude
+        
+
+class Resonator_test(Test): # LC-resonator
+    S = ((R(), "V0", "V1"),
+         (L(),"V0","V1"),
+         #(Trans_GC(nw=1), "V0", "V1"),
+         (C(),"V1",GND),
+         #(OUT, "V1",Trans_GC()("phi",1e6)),
+         (OUT, "V1"),
+         (IN, "V0"),
+         )
+
+    V = {L(): 1e-3,
+         C(): 1e-5,
+         R(): 1e+2,
+         Trans_GC(nw=1): dict(windings=[100], C = 1e-3/100**2, a = 5e-1, o = 0, n = 4),
+         "OP": [0],
+         }
+
+    result = np.array([[ 0.        ],
+       [ 1.745916  ],
+       [ 1.01468553],
+       [-3.07344884],
+       [ 3.12017221],
+       [-3.56599345],
+       [ 4.03218786],
+       [-4.01662343],
+       [ 3.60790549],
+       [-2.8083471 ]])
+
+    timespan = 0.01
+    
+    def signal(self):
+        res_freq = 1592
+        return self.sine_signal(res_freq-200)
+
+    def plot(self, p):
+        x = self.timeline()
+        y = p(self.signal())
+        pl.plot(x, y)
+        self.finish_plot(p.out_labels, timeline=x)
+
 
 class Tonestack_test(Test):
     S = ((R(4), 1, 2),
          (C(1), 1, 3),
          (C(2), 2, 4),
          (C(3), 2, 5),
-         (P(3), 6, 5, GND),
-         (P(2), 6, 4),
-         (P(1), 3, 7, 4),
+         (P(3), GND, 6, 5),
+         (P(2), None, 6, 4),
+         (P(1), 4, 3, 7),
          (IN, 1),
          (OUT, 7),
          )
@@ -302,47 +410,11 @@ class Tonestack_test(Test):
     def signal(self):
         return self.sine_signal(400)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels)
-
-
-class Resonator_test(Test): # LC-resonator
-    S = ((R(), "V0", "V1"),
-         (L(),"V0","V1"),
-         (C(),"V1",GND),
-         (OUT, "V1"),
-         (IN, "V0"),
-         )
-
-    V = {L(): 1e-3,
-         C(): 1e-5,
-         R(): 1e+2,
-         "OP": [0],
-         }
-
-    result = np.array([[ 0.        ],
-       [ 1.745916  ],
-       [ 1.01468553],
-       [-3.07344884],
-       [ 3.12017221],
-       [-3.56599345],
-       [ 4.03218786],
-       [-4.01662343],
-       [ 3.60790549],
-       [-2.8083471 ]])
-
-    timespan = 0.01
-
-    def signal(self):
-        res_freq = 1592
-        return self.sine_signal(res_freq-200)
-
-    def plot(self, y, labels):
-        x = self.timeline()
-        pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels)
 
 
 class Diode_test(Test):
@@ -370,10 +442,11 @@ class Diode_test(Test):
         a[:,0] = self.sig
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
+        y = p(self.signal())
         pl.semilogy(self.sig, -y)
         pl.plot(self.get_samples(self.sig), -self.result, "rx")
-        self.finish_plot(labels)
+        self.finish_plot(p.out_labels)
 
 
 class Diode_clipper(Test): # diode clipper
@@ -394,7 +467,6 @@ class Diode_clipper(Test): # diode clipper
          C(): 3e-7,
          "OP": [0],
          }
-    solver = dict(method='lm')
 
     result = np.array([[ -1.20000000e+00,   3.98658844e+00,  -8.01341156e-01],
        [ -9.34673367e-01,   1.60606646e+00,  -7.74066721e-01],
@@ -413,21 +485,21 @@ class Diode_clipper(Test): # diode clipper
         a[:,0] = self.sig
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
+        y = p(self.signal())
         pl.plot(self.sig, -y)
         pl.plot(self.get_samples(self.sig), -self.result, "rx")
-        self.finish_plot(labels)
+        self.finish_plot(p.out_labels)
 
 
 # class LFO_test(Test): # LFO, fail, finds wrong roots (c.f. Mačák thesis)
 
-#     OPA_ = OPA
 #     S = ((R(1), "V1", "V2"),
 #          (C(1), "V2", "V3"),
 #          (R(2), "V3", "V4"),
 #          (R(3), "V4", "V1"),
-#          (OPA_(1), GND, "V2", "V3"),# "Vee", "Vcc"),
-#          (OPA_(2), "V4", GND, "V1"),# "Vee", "Vcc"),
+#          (OPA(1), GND, "V2", "V3"),# "Vee", "Vcc"),
+#          (OPA(2), "V4", GND, "V1"),# "Vee", "Vcc"),
 #          #(IN, "Vee", "Vcc"),
 #          (OUT, "V3"),
 #          )
@@ -436,8 +508,8 @@ class Diode_clipper(Test): # diode clipper
 #          R(2): 33e3,
 #          R(3): 47e3,
 #          C(1): 5e-6,
-#          OPA_(1): 1e5,
-#          OPA_(2): 1e5,
+#          OPA(1): 1e5,
+#          OPA(2): 1e5,
 #          "OP": [],
 #          }
 
@@ -445,12 +517,12 @@ class LinOpAmp_test(Test): # linear inverting OPAMP
 
     S = ((R(2), "V-", "Vout"),
          (R(1), GND, "V-"),
-         (OPA_L(), "V+", "V-", "Vout"),
+         (OPA(), "V+", "V-", "Vout"),
          (OUT, "V-", "Vout"),
          (IN, "V+"),
          )
 
-    V = {OPA_L(): 1e5,
+    V = {OPA(): 1e5,
          R(1): 1e3,
          R(2): 1e3,
          }
@@ -471,10 +543,11 @@ class LinOpAmp_test(Test): # linear inverting OPAMP
     def signal(self):
         return self.sine_signal(500)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
 
 
 class InvOpAmp_test(Test): # inverting OPAMP
@@ -506,22 +579,23 @@ class InvOpAmp_test(Test): # inverting OPAMP
     def signal(self):
         return 12*self.sine_signal(500)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
 
 class WahWah_test(Test): # wah-wah
     S = ((R(1), "V1", "V2"),
          (C(1), "V2", "V3"),
-         (T(1), "V3", "V4", "V5"),
+         (T(1), "V4", "V3", "V5"),
          (R(4), "V4", "V6"),
-         (L(1), "V6", "V7"),
          (R(7), "V6", "V7"),
          (C(4), "V4", "V8"),
-         (R("va"), "V8", "V9"),
-         (R("vb"), "V9", GND),
-         (T(2), "V10", "V11", "V12"),
+         (P(), GND, "V8", "V9"),
+         #(R("va"), "V8", "V9"),
+         #(R("vb"), "V9", GND),
+         (T(2), "V11", "V10", "V12"),
          (C(5), "V9", "V10"),
          (R(5), "V4", "V10"),
          (R(10), "V13", "V11"),
@@ -532,8 +606,15 @@ class WahWah_test(Test): # wah-wah
          (C(2), "V6", GND),
          (R(8), "V6", GND),
          (R(6), "V7", "V3"),
-         (IN, "V1", "V13"),
+         #(L(1), "V6", 7),
+         (Trans_GC(1, nw=1), "V6", 7),
+         #(Trans_L(1, nw=1), "V6", 7),
+         (R('L'), 7, "V7"),
+         (V(), "V13"),
+         (IN, "V1"),
          (OUT, "V8"),
+         #(OUT, "V1","V2","V3","V4","V5","V6","V7","V8","V9","V10","V11","V12","V13"),
+         #(OUT, Trans_GC(1, nw=1)("W1"), Trans_GC(1, nw=1)("phi",1e3)),
          #(OUT, "V4", "V6","V7"),
          )
 
@@ -547,6 +628,7 @@ class WahWah_test(Test): # wah-wah
          R(8): 82e3,
          R(9): 10e3,
          R(10): 1e3,
+         P(): dict(value=100e3, a=5, inv=1, var='hotpotz', name='Wah'),
          R("va"): 50e3,
          R("vb"): 50e3,
          C(1): 10e-9,
@@ -555,9 +637,15 @@ class WahWah_test(Test): # wah-wah
          C(4): 220e-9,
          C(5): 220e-9,
          L(1): 500e-3,
+         R('L'): 60.,
+         # a = (4e-3/(0.5/100**2))**7
+         Trans_GC(1, nw=1): dict(windings=[100], C = 0.5/100**2, a = (8287.159*0.5/100)**7*100, o = 1e-3, n = 7),
+         Trans_L(1, nw=1): dict(windings=[100], R = 100**2/0.5),
          T(1): dict(Vt=26e-3, Is=20.3e-15, Bf=1430, Br=4),
          T(2): dict(Vt=26e-3, Is=20.3e-15, Bf=1430, Br=4),
-         "OP": [0, 8.15],
+         V(): 8.15,
+         "OP": [0],
+         "POT": dict(Pv=0.5),
          }
 
     timespan = 0.02
@@ -565,33 +653,121 @@ class WahWah_test(Test): # wah-wah
     def signal(self):
         a = self.op_signal()
         a[:,0] += 0.2*self.sine_signal(555.*1.2)[:,0]
-        #a[:,0] += 2*self.sine_signal(555.*1.0)[:,0]
+        #a[:,0] += 10*self.sine_signal(555.*1.02)[:,0]
         return a
 
     max_error = 5e-7  # presumable due to different numeric implementation of nonlinear vector function
-    result = np.array([
-        [ -1.11013509e-09],
-        [ -5.33823431e-01],
-        [  6.98293529e-01],
-        [ -3.72258478e-01],
-        [  2.72701765e-01],
-        [ -4.33033182e-01],
-        [  3.92585193e-01],
-        [ -2.60164234e-01],
-        [  2.16409909e-01],
-        [ -2.21358465e-01]])
+    result = np.array([[ -1.52955659e-09], # linear inductivity
+       [ -5.05823249e-01],
+       [  6.48561512e-01],
+       [ -4.12448688e-01],
+       [  3.39214923e-01],
+       [ -4.18385401e-01],
+       [  3.80384337e-01],
+       [ -3.10358628e-01],
+       [  2.61649974e-01],
+       [ -2.30510286e-01]])
 
-    def plot(self, y, labels):
+    result = np.array([[  3.38299440e-09], # nonlinear inductivity
+       [ -5.05702356e-01],
+       [  6.50565393e-01],
+       [ -4.16306149e-01],
+       [  3.38903677e-01],
+       [ -4.17551679e-01],
+       [  3.81734262e-01],
+       [ -3.11947188e-01],
+       [  2.61606410e-01],
+       [ -2.30657192e-01]])
+
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels, timeline=x)
+        self.finish_plot(p.out_labels, timeline=x)
+
+    def plot_spectrum(self, p, plot_variable):
+        varlist = []
+        if plot_variable:
+            if plot_variable not in p.pot_list:
+                raise ArgumentError("variable %s not found" % plot_variable)
+            ##hack
+            for k, t in self.V.items():
+                if isinstance(k, P):
+                    if not isinstance(t, dict):
+                        t = dict(value=t)
+                    var = t.get('var')
+                    if var is None:
+                        var = str(k)+'v'
+                    if var == plot_variable:
+                        break
+            loga = t.get('a', 0)
+            inv = t.get('inv', 0)
+            for i in range(5):
+                pot = i/4
+                lbl = "%s" % pot
+                if inv:
+                    pot = 1 - pot
+                if loga:
+                    pot = (math.exp(loga * pot) - 1) / (math.exp(loga) - 1)
+                varlist.append((plot_variable, pot, lbl))
+        else:
+            varlist.append((None, None, p.out_labels))
+        n = None
+        cut = None
+        def spec(y):
+            s = 20*np.log10(abs(np.fft.fft(y,n,axis=0)[cut]))
+            return np.where(s > -80, s, np.nan)
+        labels = []
+        for var, val, lbl in varlist:
+            if var is not None:
+                p.set_variable(var, val)
+            y = self.spectrum_signal(p, magnitude=1e-4)
+            if n is None:
+                n = dk_lib.pow2roundup(len(y))
+                cut = slice(n*20.0/self.FS, n*10000.0/self.FS)
+                w = fftfreq(n,1.0/self.FS)[cut]
+            pl.semilogx(w, spec(y))
+            if isinstance(lbl, basestring):
+                labels.append(lbl)
+            else:
+                labels.extend(lbl)
+        self.finish_plot(labels, loc='upper left')
+
+
+class WahWah_ss(WahWah_test): # wah-wah small signal model
+
+    result = np.array([ # RL = 0
+        [-7.06631563e-02, 3.63685251e-01, -7.61920610e-01, 8.24081826e-01, -4.77626993e-01, 1.36243627e-01, -1.37999458e-02],
+        [ 1.00000000e+00, -5.95793948e+00, 1.47953386e+01, -1.96017943e+01, 1.46127463e+01, -5.81176668e+00, 9.63415573e-01]])
+    result = np.array([ # RL = 60
+        [-7.06620964e-02, 3.63509088e-01, -7.61193126e-01, 8.22912278e-01, -4.76737510e-01, 1.35937831e-01, -1.37664640e-02],
+        [ 1.00000000e+00, -5.95545306e+00, 1.47829908e+01, -1.95772669e+01, 1.45883870e+01, -5.79967105e+00, 9.61013231e-01]])
+
+    def plot(self, p):
+        x = self.timeline()
+        y = p(self.signal())
+        pl.plot(x, y)
+        self.finish_plot(p.out_labels)
+
+    def check(self, name, args):
+        p = dk_simulator.Parser(self.S, self.V, self.FS, not args.backward_euler, create_filter=True, symbolic=False)
+        p1 = dk_simulator.Parser(self.S, self.V, self.FS, not args.backward_euler)
+        sim = dk_simulator.SimulatePy(dk_simulator.EquationSystem(p1), self.solver)
+        J = sim.jacobi()
+        f = dk_simulator.LinearFilter(p, J)
+        b, a = f.get_z_coeffs(samplerate=48000, subst_var=f.convert_variable_dict({}))
+        res = np.array([[float(v) for v in b],[float(v) for v in a]])
+        if np.allclose(res, self.result):
+            return "Ok"
+        else:
+            return "Fail"
 
 
 class Transistor_test(Test): # transitor test
     S = ((R(1), "Vcc", "Vc"),
          (R(2), "Ve", GND),
          (R(3), "Vi", "Vb"),
-         (T(1), "Vb", "Vc", "Ve"),
+         (T(1), "Vc", "Vb", "Ve"),
          (IN, "Vi", "Vcc"),
          (OUT, "Vi", "Vb", "Vc"),
          )
@@ -601,6 +777,7 @@ class Transistor_test(Test): # transitor test
          R(3): 1e6,
          T(1): dict(Vt=26e-3, Is=20.3e-15, Bf=1430, Br=4),
          "OP": [1, 10],
+         "v0": [1, 1],
          }
 
     result = np.array([[  0.00000000e+00,   5.07500000e-09,   1.00000000e+01],
@@ -620,9 +797,10 @@ class Transistor_test(Test): # transitor test
         a[:,0] += self.sig
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
+        y = p(self.signal())
         pl.plot(self.sig, y)
-        self.finish_plot(labels, timeline=self.sig)
+        self.finish_plot(p.out_labels, timeline=self.sig)
 
 
 class Triode1_test(Test): # triode test
@@ -652,13 +830,15 @@ class Triode1_test(Test): # triode test
         a[:,0] += self.sig
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
+        y = p(self.signal())
         pl.plot(self.sig, -y)
         pl.plot(self.get_samples(self.sig), -self.result, "rx")
-        self.finish_plot(labels)
+        self.finish_plot(p.out_labels)
 
 Tubes = {
     "12ax7": dict(mu = 100.0, Ex = 1.4, Kp = 600.0, Kvb = 300.0, Kg1 = 1060.0, Gco = -0.2, Gcf = 1e-5),
+    "EL34": dict(mu = 8.7, Ex = 1.35, Kp = 48.0, Kvb = 12.0, Kg1 = 1460.0, Kg2 = 4500.0, Gco = -0.2, Gcf = 1e-5),
     }
 
 class Triode2_test(Test): # triode test 2
@@ -704,19 +884,24 @@ class Triode2_test(Test): # triode test 2
         a[:,0] += self.sig
         return a
 
-    def plot(self, y, labels):
+    def plot(self, p):
+        y = p(self.signal())
         pl.plot(self.sig, y)
-        self.finish_plot(labels, loc="upper center", timeline=self.sig)
+        self.finish_plot(p.out_labels, loc="upper center", timeline=self.sig)
 
 
 class Preamp_test(Test):
-    S = ((P(6), 8, 9, GND),
+    S = ((P(6), GND, 8, 9),
+         #(R(61), 8, 9),
+         #(R(62), 9, GND),
          (V('cc3'), 18),
          (CC(2), 11, 13),
          (V('cc2'), 12),
          (Triode(2), 9, 11, 10),
          (V('cc1'), 5),
          (CC(1), 4, 6),
+         (R(71), 6, GND),
+         (R(72), 13, GND),
          (Triode(3), 15, 17, 16),
          (Triode(1), 2, 4, 3),
          (C('m3'), 15, 17),
@@ -740,7 +925,7 @@ class Preamp_test(Test):
          (R(3), 3, GND),
          (R(2), 2, 1),
          (R(1), 1, GND),
-         (OUT, 17),
+         (OUT, Out(17,2.62e-3)),
          (IN, 1),
          )
 
@@ -754,6 +939,8 @@ class Preamp_test(Test):
          C(5): 1.e-9,
          C(6): 22.e-9,
          C(7): 470.e-12,
+         R(71): 1.e12,
+         R(72): 1.e12,
          R(1): 1.e6,
          R(10): 470.e3,
          R(11): 820.,
@@ -766,6 +953,8 @@ class Preamp_test(Test):
          R(8): 100.e3,
          R(9): 470.e3,
          P(6): 1.e6,
+         R(61): 0.5e6,
+         R(62): 0.5e6,
          Triode(1): Tubes['12ax7'],
          Triode(2): Tubes['12ax7'],
          Triode(3): Tubes['12ax7'],
@@ -774,7 +963,7 @@ class Preamp_test(Test):
          V('cc3'): 385.,
          }
 
-    result = np.array(
+    result = 2.62e-3 * np.array(
         [[ 221.71522314],
          [ 171.55513962],
          [ 384.87040955],
@@ -791,7 +980,8 @@ class Preamp_test(Test):
     def signal(self):
         return 0.15*self.sine_signal(150.0)
 
-    def plot(self, y, labels):
+    def plot(self, p):
         x = self.timeline()
+        y = p(self.signal())
         pl.plot(x, y)
-        self.finish_plot(labels)
+        self.finish_plot(p.out_labels, timeline=x)
